#ifndef DIFFREN_H
#define DIFFREN_H

#include "diffren_math.h"

/* Mark as unused to silence compiler warnings about unused parameters */
#define UNUSED(x) ((void)sizeof((x)))

/* Handles rendering of scenes */
typedef struct DeferredRenderer DeferredRenderer;

/* Opaque handle for an imported 3d model */
typedef struct Model Model;

/* Create the deferred renderer.
 * w and h are the size of the gbuffer in pixels. 
 * near and far define the near and far plane distance. */
DeferredRenderer *CreateDeferredRenderer(int w, int h, float near, float far);

/* Free all resources allocated for the deferred renderer. */
void DestroyDeferredRenderer(DeferredRenderer *renderer);

void BeginFrame(DeferredRenderer *renderer, V3 eye_pos, V3 center, V3 up, V3 ambient_color);

void AddLight(DeferredRenderer *renderer, V3 center, V3 color, float radius);

/* Render a model into the scene */
void RenderModel(DeferredRenderer *renderer, const Model *model, M4 transform);

/* Resolve the geometry buffer and lights into a single rendered image */
void ResolveGBuffer(DeferredRenderer *renderer);

void ToneMapping(DeferredRenderer *renderer);

void Differentiate(DeferredRenderer *renderer);

/* Import a 3d model from the given file */
Model *ImportModel(const char *file);

/* Destroy an imported model */
void DestroyModel(Model *model);

#endif
