#ifndef DIFFREN_MODEL_H
#define DIFFREN_MODEL_H

/* 3D model structures */

#include <glad/gl.h>
#include <stdint.h>

#include "diffren_math.h"

typedef struct {
    V3 position;
    V3 normal;
    V3 tangent;
    V2 texcoord;
} Vertex;

typedef struct {
    int width;
    int height;
    int channels;

    char *name;

    /* OpenGL objects */
    GLuint tex;
} Texture;

typedef struct {
    Texture *albedo;
    Texture *normal;
    Texture *orm;
} Material;

typedef struct {
    Vertex *vertices;
    uint32_t *indices;
    unsigned int vertex_count;
    unsigned int index_count;

    Material *material;

    /* OpenGL objects */
    GLuint vbo;
    GLuint ebo;
    GLuint vao;
} Mesh;

typedef struct Model {
    Mesh *meshes;
    unsigned int mesh_count;

    Material *materials;
    unsigned int material_count;

    Texture *textures;
    unsigned int texture_count;

    char *directory;
} Model;

#endif
